/*
 * the parts of blt.h and bltInt.h necessary to extract the busy widget
 */

/*
 * blt.h --
 *
 * Copyright 1991-1998 by Bell Labs Innovations for Lucent
 * Technologies.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that the copyright notice and warranty
 * disclaimer appear in supporting documentation, and that the names
 * of Lucent Technologies any of their entities not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 *
 * Lucent Technologies disclaims all warranties with regard to this
 * software, including all implied warranties of merchantability and
 * fitness.  In no event shall Lucent Technologies be liable for any
 * special, indirect or consequential damages or any damages
 * whatsoever resulting from loss of use, data or profits, whether in
 * an action of contract, negligence or other tortuous action, arising
 * out of or in connection with the use or performance of this
 * software.
 */

#include <assert.h>

#ifndef _ANSI_ARGS_
#   define _ANSI_ARGS_(x)       ()
#endif

#ifdef __cplusplus
#define ANYARGS (...)
#else
#define ANYARGS ()
#endif

/*
 * ----------------------------------------------------------------------
 *
 * Blt_OpProc --
 *
 * 	Generic function prototype of CmdOptions.
 *
 * ----------------------------------------------------------------------
 */

typedef int (*Blt_OpProc) _ANSI_ARGS_(ANYARGS);

/*
 * ----------------------------------------------------------------------
 *
 * Blt_OpSpec --
 *
 * 	Structure to specify a set of operations for a Tcl command.
 *      This is passed to the Blt_GetOperation procedure to look
 *      for a function pointer associated with the operation name.
 *
 * ----------------------------------------------------------------------
 */
typedef struct {
    char *name;			/* Name of operation */
    int minChars;		/* Minimum # characters to disambiguate */
    Blt_OpProc proc;
    int minArgs;		/* Minimum # args required */
    int maxArgs;		/* Maximum # args required */
    char *usage;		/* Usage message */

} Blt_OpSpec;

typedef enum {
    BLT_OPER_ARG0,		/* Operation is the first argument. */
    BLT_OPER_ARG1,		/* Operation is the second argument. */
    BLT_OPER_ARG2,		/* Operation is the third argument. */
    BLT_OPER_ARG3,		/* Operation is the fourth argument. */
    BLT_OPER_ARG4		/* Operation is the fifth argument. */
} Blt_OpIndex;
