# busy widget - based initially on BLT busy 2.4q
# 
# $Id$

set tcl_prefix [file normalize /usr/local//src/tcl]   ;# will become a cmdline flag
set tk_prefix  [file normalize /usr/local//src/tk]    ;# will become a cmdline flag

if {[catch {package require critcl}] \
            && !([critcl::done] || [critcl::compiling])} {
    puts stderr "This extension cannot be compiled without critcl enabled"
    exit 1
}

package provide busy 1.0

critcl::tk

# critcl doesn't provide Tcl and Tk internal headers (required by
# busy), nor X11 libraries to link agains, so you need to adjust
# the lines below as appropriate

if {$tcl_platform(os) eq "OSF1"} {
    critcl::cheaders -I/usr/sys/include/sys
}

if {[critcl::check "#include <limits.h>"]} {
    critcl::cheaders -DHAVE_LIMITS_H
}
if {[critcl::check "#include <unistd.h>"]} {
    critcl::cheaders -DHAVE_UNISTD_H
} 
critcl::cheaders -DNDEBUG

# critcl doesn't provide Tcl and Tk internal headers
critcl::cheaders -I$tcl_prefix/generic
critcl::cheaders -I$tk_prefix/generic
critcl::cheaders -I.

#
#   platform specific declarations
#

switch $tcl_platform(platform) {
    unix {
        switch $tcl_platform(os) {
            Darwin  {
                set xinclude /usr/X11R6/include
                set xlib /usr/X11R6/lib
            }
            SunOS {
                set xinclude /usr/openwin/include
                set xlib /usr/openwin/lib
            }
            default {
                set xinclude /usr/X11R6/lib
                set xlib $xinclude
            }
        }
        critcl::cheaders -I$xinclude
        critcl::clibraries -L$xlib -lX11
    }
    windows {
        critcl::cheaders -DWIN32 -I$tk_prefix/win -I$tk_prefix/xlib
    }
    default {
        puts stderr \
            "this extension hasn't been ported to $tcl_platform(platform)"
        exit 1
    }
}

# arrange for copy of this script to be included (to get the Tcl procs)
critcl::tsources busy.tcl

critcl::ccode {
    #include "busy.h"
    ThreadData *BusyDataPtr = NULL;

    /* convert objv to argv */
    char **
    obj_convert(int objc, Tcl_Obj *CONST objv[])
    {
        int i;
        char **argv = (char **) Tcl_Alloc((objc+1) * sizeof (char *));
        for (i = 0; i < objc; i++) {
            argv[i] = Tcl_GetString(objv[i]);
        }
        return argv;
    }
}

namespace eval busy {
    namespace export cget configure forget hold isbusy release status windows
    if {[string equal $::tcl_platform(platform) windows]} {
        critcl::csources winutil.c
        set cursor wait
    } else {
        set cursor watch
    }
    critcl::ccode {
        #include "busy.c"
        #include "window.c"
    }

    critcl::cheaders "-DDEF_BUSY_CURSOR=\"$cursor\""

    critcl::cinit {
        BusyDataPtr = (ThreadData *) GetBusyCmdData(ip);
    } {}

    proc WrongNumArgs {msg} {
        error "wrong # args: should be \"$msg\""
    }

    # Tcl procs - parse arguments and call C functions to do the work

    proc cget {args} {
        if {[llength $args] != 2} {
            WrongNumArgs "busy::cget window option"
        }
        eval c_cget $args
    }

    proc configure {args} {
        if {[llength $args] < 1} {
            WrongNumArgs "busy::configure window ?options?"
        }
        eval c_configure $args
    }

    proc hold {args} {
        set argc [llength $args]
        if {$argc != 1 && $argc != 3} {
            WrongNumArgs "busy::hold window ?-cursor name?"
        }
        eval c_hold $args
    }

    proc status {args} {
        if {[llength $args] != 1} {
            WrongNumArgs "busy::status window"
        }
        eval c_status $args
    }

    # these don't need Tcl front-ends

    critcl::ccode {
        /* Forward declarations */
        void DestroyBusy _ANSI_ARGS_((DestroyData dataPtr));
    }
        
    critcl::ccommand forget {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Busy *busyPtr;
        int i;

        for (i = 1; i < argc; i++) {
            if (GetBusy(dataPtr, ip, argv[i], &busyPtr) != TCL_OK) {
                return TCL_ERROR;
            }
            /* Unmap the window even though it will be soon destroyed */
            HideBusyWindow(busyPtr);
            Tcl_EventuallyFree((ClientData)busyPtr, DestroyBusy);
        }
        Tcl_SetResult(ip, NULL, NULL);
        return TCL_OK;
    }

    critcl::ccommand isbusy {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Busy *busyPtr;
        Tcl_HashEntry *hPtr;
        Tcl_HashSearch cursor;

        for (hPtr = Tcl_FirstHashEntry(&(dataPtr->busyTable), &cursor);
            hPtr != NULL; hPtr = Tcl_NextHashEntry(&cursor)) {
            busyPtr = (Busy *)Tcl_GetHashValue(hPtr);
            if (busyPtr->interp == ip) {
                if ((busyPtr->isBusy) && ((argc != 2) ||
                    (Tcl_StringMatch(Tk_PathName(busyPtr->client), argv[1])))) {
                    Tcl_AppendElement(ip, Tk_PathName(busyPtr->client));
                }
            }
        }
    }

    critcl::ccommand release {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Busy *busyPtr;
        int i;

        for (i = 1; i < argc; i++) {
            if (GetBusy(dataPtr, ip, argv[i], &busyPtr) != TCL_OK) {
                return TCL_ERROR;
            }
            HideBusyWindow(busyPtr);
            busyPtr->isBusy = FALSE;
        }
        Tcl_SetResult(ip, NULL, NULL);
        return TCL_OK;
    }

    critcl::ccommand windows {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Tcl_HashEntry *hPtr;
        Tcl_HashSearch cursor;
        Busy *busyPtr;

        for (hPtr = Tcl_FirstHashEntry(&(dataPtr->busyTable), &cursor);
            hPtr != NULL; hPtr = Tcl_NextHashEntry(&cursor)) {
            busyPtr = (Busy *)Tcl_GetHashValue(hPtr);
            if (busyPtr->interp == ip) {
                if ((argc != 2) ||
                    (Tcl_StringMatch(Tk_PathName(busyPtr->client), argv[1]))) {
                    Tcl_AppendElement(ip, Tk_PathName(busyPtr->client));
                }
            }
        }
        return TCL_OK;
    }

    # C functions to do the hard work

    critcl::ccommand c_cget {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Busy *busyPtr;
        int result;

        if (GetBusy(dataPtr, ip, argv[1], &busyPtr) != TCL_OK) {
            return TCL_ERROR;
        }
        Tcl_Preserve((ClientData)busyPtr);
        result = Tk_ConfigureValue(ip, busyPtr->client, configSpecs,
                        (char *)busyPtr, argv[2], 0);
        Tcl_Release((ClientData)busyPtr);
        return result;
    }

    critcl::ccommand c_configure {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Busy *busyPtr;
        int result;

        if (GetBusy(dataPtr, ip, argv[1], &busyPtr) != TCL_OK) {
            return TCL_ERROR;
        }
        Tcl_Preserve((ClientData)busyPtr);
        if (argc == 2) {
            result = Tk_ConfigureInfo(ip, busyPtr->client, configSpecs,
                (char *)busyPtr, (char *)NULL, 0);
        } else if (argc == 3) {
            result = Tk_ConfigureInfo(ip, busyPtr->client, configSpecs,
                (char *)busyPtr, argv[2], 0);
        } else {
            result = ConfigureBusy(ip, busyPtr, argc - 2, argv + 2);
        }
        Tcl_Release((ClientData)busyPtr);
        return result;
    }

    critcl::ccommand c_hold {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Tk_Window tkwin;
        Tcl_HashEntry *hPtr;
        Busy *busyPtr;
        int isNew, result;

        tkwin = Tk_NameToWindow(ip, argv[1], Tk_MainWindow(ip));
        if (tkwin == NULL) {
            return TCL_ERROR;
        }
        hPtr = Tcl_CreateHashEntry(&(dataPtr->busyTable),(char *)tkwin, &isNew);
        if (isNew) {
            busyPtr = (Busy *)CreateBusy(ip, tkwin);
            if (busyPtr == NULL) {
                return TCL_ERROR;
            }
            Tcl_SetHashValue(hPtr, (char *)busyPtr);
            busyPtr->hashPtr = hPtr;
        } else {
            busyPtr = (Busy *)Tcl_GetHashValue(hPtr);
        }
        result = ConfigureBusy(ip, busyPtr, argc - 2, argv + 2);
        /* Don't map the busy window unless the reference window is also
         * displayed */
        if (Tk_IsMapped(busyPtr->client)) {
            ShowBusyWindow(busyPtr);
        } else {
            HideBusyWindow(busyPtr);
        }
        busyPtr->isBusy = TRUE;
        Tcl_Free(argv);
        Tcl_SetResult(ip, NULL, NULL);
        return TCL_OK;
    }

    critcl::ccommand c_status {data ip objc objv} -clientdata BusyDataPtr {
        int argc = objc;
        char **argv = obj_convert(objc, objv);
        ThreadData *dataPtr = (ThreadData *) data;
        Busy *busyPtr;

        if (GetBusy(dataPtr, ip, argv[1], &busyPtr) != TCL_OK) {
            return TCL_ERROR;
        }
        Tcl_Preserve((ClientData)busyPtr);
        Tcl_SetResult(ip, busyPtr->isBusy ? "1" : "0", TCL_STATIC);
        Tcl_Release((ClientData)busyPtr);
        return TCL_OK;
    }

}

if {0 && [critcl::failed]} {
    # note that calling critcl::failed stops the compile errors being
    # displayed on stderr
    puts stderr "compile failed - check in [critcl::cache]  for errors"
    exit 1
}
